@@ -1,5 +1,5 @@
 /*
-Copyright (C) 1997-2009 Erwin Waterlander
+Copyright (C) 1997-2011 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,60 +22,115 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #include <stdlib.h>
 #include <signal.h>
 #include "wcd.h"
-#ifdef WCD_UNICODE
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
 #ifndef __USE_XOPEN
 #define __USE_XOPEN
 #endif
 #include <wchar.h>
 #endif
-#ifdef WCD_UTF16
-#include <windows.h>
-#endif
+#include "display.h"
 #include "std_macr.h"
 #include "structur.h"
 #include "nameset.h"
-#include "display.h"
 #include "config.h"
 #include "dosdir.h"
 
 #include <stdarg.h>
+#if defined(WCD_WINDOWS)
+#include <windows.h>
+#endif
 
 
 
+#ifdef WCD_ANSI
+/* wide char to UTF-8 */
+size_t wcstoansi(char *mbstr, const wchar_t *wcstr, int len)
+{
+   return((size_t)(WideCharToMultiByte(CP_ACP, 0, wcstr, -1, mbstr, len, NULL, NULL) -1));
+}
+size_t ansitowcs(wchar_t *wcstr, const char *mbstr, int len)
+{
+   return((size_t)(MultiByteToWideChar(CP_ACP, 0, mbstr, -1, wcstr, len) -1));
+}
+#endif
 #ifdef WCD_UTF16
 /* wide char to UTF-8 */
-int wcstoutf8(char *mbstr, wchar_t *wcstr, int len)
+size_t wcstoutf8(char *mbstr, const wchar_t *wcstr, int len)
 {
-   return(WideCharToMultiByte(CP_UTF8, 0, wcstr, -1, mbstr, len, NULL, NULL) -1);
+   return((size_t)(WideCharToMultiByte(CP_UTF8, 0, wcstr, -1, mbstr, len, NULL, NULL) -1));
 }
-int utf8towcs(wchar_t *wcstr, char *mbstr, int len)
+size_t utf8towcs(wchar_t *wcstr, const char *mbstr, int len)
 {
-   return(MultiByteToWideChar(CP_UTF8, 0, mbstr, -1, wcstr, len) -1);
+   return((size_t)(MultiByteToWideChar(CP_UTF8, 0, mbstr, -1, wcstr, len) -1));
 }
 #endif
+/*
+ * wcd_printf()  : printf wrapper, print in Windows Console in Unicode mode,
+ *                 to have consistent output. Regardless of active code page.
+ *
+ * Windows Unicode: Print Windows ANSI encoded format with UTF-8 encoded arguments.
+ * Windows ANSI   : Print Windows ANSI encoded format and arguments.
+ * All others     : Use standard printf.
+ *
+ * On Windows the file system uses always Unicode UTF-16 encoding,
+ * regardless of the system default code page. This means
+ * that files and directories can have names that
+ * can't be encoded in the default system Windows ANSI code page.
+ *
+ * The Windows console supports printing of any UTF-16 wide character,
+ * regardless of code page, via WriteConsoleW().
+ *
+ * Wcd for Windows with Unicode support translates all UTF-16 directory
+ * names to UTF-8, to be able to work with char type strings.
+ * This is done to keep the code portable.
+ *
+ * Wcd's messages are encoded in the default Windows ANSI code page,
+ * which can be translated with gettext. Gettext/libintl recodes
+ * messages (format) to the system default ANSI code page.
+ */
 
-/* Print an UTF-8 multi-byte string */
 void wcd_printf( const char* format, ... ) {
    va_list args;
-#ifdef WCD_UTF16
+#if defined(_WIN32) && !defined(__CYGWIN__) /* Windows, not Cygwin */
    wchar_t wstr[DD_MAXPATH];
    char buf[DD_MAXPATH];
+#  ifdef WCD_UTF16
+   char formatmbs[DD_MAXPATH];
+   wchar_t formatwcs[DD_MAXPATH];
+#  endif
+   HANDLE stduit;
 
-   HANDLE stduit =GetStdHandle(STD_OUTPUT_HANDLE);
-#endif
+   va_start(args, format);
+   stduit =GetStdHandle(STD_OUTPUT_HANDLE);
 
-   va_start( args, format );
-#ifdef WCD_UTF16
-   /* Assume the multi-byte string is in UTF-8 encoding. */
-   vsnprintf( buf, sizeof(wstr), format, args);
+#  ifdef WCD_UTF16  /* Wcd for Windows with Unicode support */
+   /* The format string is encoded in the system default
+    * Windows ANSI code page. May have been translated
+    * by gettext. Convert it to wide characters. */
+   MultiByteToWideChar(CP_ACP,0, format, -1, formatwcs, DD_MAXPATH);
+   /* then convert the format string to UTF-8 */
+   WideCharToMultiByte(CP_UTF8, 0, formatwcs, -1, formatmbs, DD_MAXPATH, NULL, NULL);
+
+   /* Assume the arguments (directory names) are in UTF-8 encoding, because
+    * in Windows Unicode mode all treedata files are written in UTF-8 format.
+    * Print to buffer (UTF-8) */
+   vsnprintf( buf, sizeof(buf), formatmbs, args);
+    /* Convert UTF-8 buffer to wide characters, and print to console. */
    if (MultiByteToWideChar(CP_UTF8,0, buf, -1, wstr, DD_MAXPATH) > 0  )
+#  else
+   /* Everything is in ANSI code page */
+   vsnprintf( buf, sizeof(buf), format, args);
+   if (MultiByteToWideChar(CP_ACP,0, buf, -1, wstr, DD_MAXPATH) > 0  )
+#  endif
       WriteConsoleW(stduit, wstr, wcslen(wstr), NULL, NULL);
    else
    {
+      /* An error occured. */
       printf("A:");
       vprintf( format, args );
    }
 #else
+   va_start(args, format);
    vprintf( format, args );
 #endif
    va_end( args );
@@ -83,31 +138,32 @@ void wcd_printf( const char* format, ... ) {
 
 
 /*
- * int str_columns (char *s)
+ * size_t str_columns (char *s)
  *
  * Return number of colums a string takes. In case
  * of erroneous multi-byte sequence, return number of
  * 8 bit characters.
  */
 
-int str_columns (char *s)
+size_t str_columns (char *s)
 {
-#ifdef WCD_UNICODE
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
    static wchar_t wstr[DD_MAXPATH];
-   int i;
+   size_t i;
+   int j;
 
    /* convert to wide characters. i = nr. of characters */
-   i= MBSTOWCS(wstr,s,DD_MAXPATH);
-   if ( i < 0)
+   i= MBSTOWCS(wstr,s,(size_t)DD_MAXPATH);
+   if ( i == (size_t)(-1))
       return(strlen(s));
    else
    {
-      i = wcswidth(wstr,DD_MAXPATH);
-      /* i =  nr. of columns */
-      if ( i < 0)
+      j = wcswidth(wstr,(size_t)DD_MAXPATH);
+      /* j =  nr. of columns */
+      if ( j < 0)
          return(strlen(s));
       else
-         return(i);
+         return((size_t)j);
    }
 #else
    return(strlen(s));
@@ -133,10 +189,10 @@ void swap(nameset list, int i, int j)
 void ssort (nameset list, int left, int right)
 {
  int i, last;
-#ifdef WCD_UNICODE
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
  static wchar_t wstr_left[DD_MAXPATH];
  static wchar_t wstr_right[DD_MAXPATH];
- int len1,len2;
+ size_t len1,len2;
 #endif
 
   if (left >= right) return; /* fewer than 2 elements */
@@ -146,10 +202,10 @@ void ssort (nameset list, int left, int right)
 
   for (i = left+1; i <=right; i++)
   {
-#ifdef WCD_UNICODE
-   len1 = MBSTOWCS(wstr_left, list->array[left],DD_MAXPATH);
-   len2 = MBSTOWCS(wstr_right,list->array[i],DD_MAXPATH);
-   if ((len1<0)||(len2<0))
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
+   len1 = MBSTOWCS(wstr_left, list->array[left],(size_t)DD_MAXPATH);
+   len2 = MBSTOWCS(wstr_right,list->array[i],(size_t)DD_MAXPATH);
+   if ((len1 == (size_t)(-1)) || (len2 == (size_t)(-1)))
    {
       /* Erroneous multi-byte sequence */
       /* Try 8 bit characters */
@@ -180,7 +236,7 @@ void ssort (nameset list, int left, int right)
 
 void sort_list(nameset list)
 {
- ssort(list,0,(list->size)-1);
+ ssort(list,0,(int)(list->size)-1);
 }
 
 /************************************************************************
@@ -191,9 +247,9 @@ void sort_list(nameset list)
  ************************************************************************/
 
 #if (defined(WCD_USECONIO) || defined(WCD_USECURSES))
-int maxLength(nameset list)
+size_t maxLength(nameset list)
 {
-   int i,len,maxlen = 0;
+   size_t i, len, maxlen = 0;
 
    if (list == NULL)
    {
@@ -211,9 +267,9 @@ int maxLength(nameset list)
    else
       return 32 ;     /* minimal width for help screen */
 }
-int maxLengthStack(WcdStack s)
+size_t maxLengthStack(WcdStack s)
 {
-   int i,len,maxlen = 0;
+   size_t i, len,maxlen = 0;
 
    if (s == NULL)
    {
@@ -819,13 +875,13 @@ void signalSigwinchDisplay (int sig)
 
 void wcd_mvwaddstr(WINDOW *win, int x, int y, char *str)
 {
-#ifdef WCD_UNICODE
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
    static wchar_t wstr[DD_MAXPATH];
-   int i;
+   size_t i;
 
    /* convert to wide characters. i = nr. of characters */
-   i= mbstowcs(wstr,str,DD_MAXPATH);
-   if ( i < 0)
+   i= mbstowcs(wstr,str,(size_t)DD_MAXPATH);
+   if ( i == (size_t)(-1))
    {
       /* Erroneous multi-byte sequence */
       /* Try 8 bit characters */
@@ -841,8 +897,9 @@ void wcd_mvwaddstr(WINDOW *win, int x, int y, char *str)
 void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbers)
 {
    wcd_uchar *s;
-   int len, j, nr_offset;
-#ifdef WCD_UNICODE
+   size_t len;
+   int j, nr_offset;
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
    static wchar_t wstr[DD_MAXPATH];
    int width, c;
 #endif
@@ -851,8 +908,8 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
 
    if (s != NULL)
    {
-#ifdef WCD_UNICODE
-      len = MBSTOWCS(wstr,(char *)s,DD_MAXPATH); /* number of wide characters */
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
+      len = MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
 #else
       len = strlen((char *)s);
 #endif
@@ -861,31 +918,31 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
       else
          nr_offset = 3;
 
-      wmove(win,y,nr_offset);
+      wmove(win,y,(int)nr_offset);
 
-#ifdef WCD_UNICODE
-      if (len<0)
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
+      if (len == (size_t)(-1))
       {
          /* Erroneous multi-byte sequence */
          /* Try 8 bit characters */
          len = strlen((char *)s);
-         for(j=xoffset;(j<len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
+         for(j=xoffset;(j<(int)len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
          {
-            waddch(win,s[j]);
+            waddch(win,(chtype)s[j]);
          }
       } else {
          c = 0; /* count characters with width > 0 from beginning of string. */
          j = 0;
-         while ((j<len)&&(c<xoffset))
+         while ((j<(int)len)&&(c<xoffset))
          {
             if (wcwidth(wstr[j]) != 0 )
                c++;
             j++; /* j advances over combining characters */
          }
-         while ((j<len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
+         while ((j<(int)len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
            j++;
          width = wcwidth(wstr[j]);
-         while ((j<len)&&((nr_offset+width)<(COLS-1)))
+         while ((j<(int)len)&&((nr_offset+width)<(COLS-1)))
          {
             waddnwstr(win,wstr+j,1);
             j++;
@@ -893,9 +950,9 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
          }
       }
 #else
-      for(j=xoffset;(j<len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
+      for(j=xoffset;(j<(int)len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
       {
-         waddch(win,s[j]);
+         waddch(win,(chtype)s[j]);
       }
 #endif
    }
@@ -904,8 +961,9 @@ void printLine(WINDOW *win, nameset n, int i, int y, int xoffset, int *use_numbe
 void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *use_numbers)
 {
    wcd_uchar *s;
-   int len, j, nr_offset;
-#ifdef WCD_UNICODE
+   size_t len;
+   int j, nr_offset;
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
    static wchar_t wstr[DD_MAXPATH];
    int width, c;
 #endif
@@ -914,8 +972,8 @@ void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *us
 
    if (s != NULL)
    {
-#ifdef WCD_UNICODE
-      len = MBSTOWCS(wstr,(char *)s,DD_MAXPATH); /* number of wide characters */
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
+      len = MBSTOWCS(wstr,(char *)s,(size_t)DD_MAXPATH); /* number of wide characters */
 #else
       len = strlen((char *)s);
 #endif
@@ -926,31 +984,31 @@ void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *us
 
       wmove(win,y,nr_offset);
 
-#ifdef WCD_UNICODE
-      if (len<0)
+#if defined(WCD_UNICODE) || defined(WCD_WINDOWS)
+      if (len == (size_t)(-1))
       {
          /* Erroneous multi-byte sequence */
          /* Try 8 bit characters */
          len = strlen((char *)s);
-         for(j=xoffset;(j<len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
+         for(j=xoffset;(j<(int)len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
          {
-            waddch(win,s[j]);
+            waddch(win,(chtype)s[j]);
          }
          if ((i == ws->current) && ((nr_offset+j-xoffset+2)<(COLS-1)))
             wprintw(win," *");
       } else {
          c = 0; /* count characters with width > 0 from beginning of string. */
          j = 0;
-         while ((j<len)&&(c<xoffset))
+         while ((j<(int)len)&&(c<xoffset))
          {
             if (wcwidth(wstr[j]) != 0 )
                c++;
             j++; /* j advances over combining characters */
          }
-         while ((j<len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
+         while ((j<(int)len)&&(wcwidth(wstr[j]) == 0 ))  /* Skip combining characters */
            j++;
          width = wcwidth(wstr[j]);
-         while ((j<len)&&((nr_offset+width)<(COLS-1)))
+         while ((j<(int)len)&&((nr_offset+width)<(COLS-1)))
          {
             waddnwstr(win,wstr+j,1);
             j++;
@@ -960,9 +1018,9 @@ void printStackLine(WINDOW *win, WcdStack ws, int i, int y, int xoffset, int *us
             wprintw(win," *");
       }
 #else
-      for(j=xoffset;(j<len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
+      for(j=xoffset;(j<(int)len)&&((nr_offset+j-xoffset)<(COLS-1));j++)
       {
-         waddch(win,s[j]);
+         waddch(win,(chtype)s[j]);
       }
       if ((i == ws->current) && ((nr_offset+j-xoffset+2)<(COLS-1)))
          wprintw(win," *");
@@ -996,7 +1054,7 @@ void print_list_stack(WINDOW *scrollWin, int lines_per_page,int line, WcdStack w
    {
       for (i=top;i<=bottom;i++)
       {
-        j = (i + start)%(ws->size);
+        j = (i + start)%(int)(ws->size);
         mvwprintw(scrollWin,line,0,"%c ",(char)(((i-top)%lines_per_page) + 'a'));
         printStackLine(scrollWin, ws, j, line, xoffset, &use_numbers);
         line++;
@@ -1006,7 +1064,7 @@ void print_list_stack(WINDOW *scrollWin, int lines_per_page,int line, WcdStack w
    {
       for (i=top;i<=bottom;i++)
       {
-        j = (i + start)%(ws->size);
+        j = (i + start)%(int)(ws->size);
         /* mvwprintw(scrollWin,line,0,"%d  %s",i + 1,ws->dir[j]); */
         mvwprintw(scrollWin,line,0,"%2d ",(i-top)%lines_per_page + 1);
         printStackLine(scrollWin, ws, j, line, xoffset, &use_numbers);
@@ -1066,7 +1124,7 @@ void displayRefresh(int init)
    page = wcd_display.bottom / wcd_display.lines_per_page + 1 ;
 
    sprintf(buf,_(" w=up x=down ?=help  Page %d/%d "),page,(wcd_display.size -1)/wcd_display.lines_per_page +1);
-   pageoffset = COLS - str_columns(buf);
+   pageoffset = COLS - (int)str_columns(buf);
    if (pageoffset < 0)
       pageoffset = 0;
    wmove (wcd_display.inputWin, 0, pageoffset);
@@ -1074,10 +1132,10 @@ void displayRefresh(int init)
 
    sprintf(buf,_("Please choose one (<Enter> to abort): "));
    wcd_mvwaddstr(wcd_display.inputWin,2,0,buf);
-   offset = str_columns(buf) ;
+   offset = (int)str_columns(buf) ;
    wmove (wcd_display.inputWin, 2, offset);
    waddstr(wcd_display.inputWin, wcd_display.number_str);
-   n = str_columns(wcd_display.number_str) ;
+   n = (int)str_columns(wcd_display.number_str) ;
    wmove (wcd_display.inputWin, 2, offset + n);
 
    prefresh(wcd_display.scrollWin,0,0,0,0,wcd_display.scrollWinHeight-1,COLS-1);
@@ -1113,7 +1171,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 {
   int i, n=0, c=0;
   int displayed_list;
-  int len ;
+  size_t len ;
 #ifndef __PDCURSES__
   SCREEN *sp;
 #endif
@@ -1132,18 +1190,18 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
    if (list != NULL)    /* normal list */
    {
       sort_list(list);
-      wcd_display.size = list->size;
+      wcd_display.size = (int)list->size;
    }
    else
       if (ws != NULL)   /* stack */
       {
-         if( ((ws->size) <= 0) || ((ws->size) > ws->maxsize) || ((ws->size) > ws->maxsize) )
+         if( ((int)(ws->size) == 0) || ((int)(ws->size) > ws->maxsize) )
             return(WCD_ERR_LIST); /* in case stack file was corrupt */
          else
          {
-            wcd_display.size = ws->size;
+            wcd_display.size = (int)ws->size;
 
-            if (ws->size < ws->maxsize)
+            if ((int)ws->size < ws->maxsize)
                wcd_display.start = 0;
             else
                wcd_display.start = ws->lastadded + 1;
@@ -1297,7 +1355,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
          break;
       case '.':
       case KEY_RIGHT:
-         if (wcd_display.shift < len)
+         if (wcd_display.shift < (int)len)
             wcd_display.shift++;
          break;
        case '<':
@@ -1309,8 +1367,8 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
       case ']':
       case '>':
          wcd_display.shift +=10;
-         if (wcd_display.shift > len)
-            wcd_display.shift=len;
+         if (wcd_display.shift > (int)len)
+            wcd_display.shift=(int)len;
          break;
       case Key_CTRL ('a'):
       case KEY_HOME:
@@ -1320,7 +1378,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 #ifdef KEY_END
       case KEY_END:
 #endif
-         wcd_display.shift = len - COLS/2;
+         wcd_display.shift = (int)len - COLS/2;
           if (wcd_display.shift < 0)
              wcd_display.shift=0;
          break;
@@ -1387,13 +1445,13 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 
    if ((ws != NULL)&&(list == NULL)) /* stack */
    {
-      if (( i <=0)||(i > ws->size)) /* fail */
+      if (( i <=0)||(i > (int)ws->size)) /* fail */
       {
         return(WCD_ERR_LIST);
        }
       else    /* succes */
       {
-        i = ( i - 1 + wcd_display.start)%(ws->size);
+        i = ( i - 1 + wcd_display.start)%(int)(ws->size);
         ws->current = i;
       }
    }
@@ -1407,6 +1465,7 @@ int display_list_curses(nameset list, WcdStack ws, int perfect,int use_numbers)
 int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
 {
    int i;
+   size_t ii;
    int k, start, j;
 
    if (list != NULL) /* normal list */
@@ -1415,13 +1474,13 @@ int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
 
       if ( use_stdout & WCD_STDOUT_DUMP )
       {
-         for (i=0;i<list->size;i++)
-            wcd_printf("%s\n", list->array[i]);
+         for (ii=0;ii<list->size;ii++)
+            wcd_printf("%s\n", list->array[ii]);
       }
       else
       {
-         for (i=0;i<list->size;i++)
-            printf("%d  %s\n",i+1,list->array[i]);
+         for (ii=0;ii<list->size;ii++)
+            wcd_printf("%lu  %s\n",(unsigned long)(ii+1),list->array[ii]);
       }
 
       if ( use_stdout & WCD_STDOUT_DUMP )
@@ -1432,7 +1491,7 @@ int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
            printf(_("\nPerfect "));
          else
            printf(_("\nWild "));
-         printf(_("match for %d directories.\n"),list->size);
+         printf(_("match for %lu directories.\n"),(unsigned long)list->size);
          printf(_("Please choose one (<Enter> to abort): "));
       }
 
@@ -1445,12 +1504,12 @@ int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
       if(ws->maxsize <= 0)
          return (WCD_ERR_LIST);
       else
-         if( ((ws->size) <= 0) || ((ws->size) > ws->maxsize) )
+         if( ((ws->size) == 0) || ((ws->size) > (size_t)ws->maxsize) )
          return (WCD_ERR_LIST);
          else
          {
 
-            if (ws->size < ws->maxsize)
+            if (ws->size < (size_t)ws->maxsize)
                 start = 0;
              else
                  start = ws->lastadded + 1;
@@ -1459,14 +1518,14 @@ int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
                  start = 0;
 
             k=1;
-            for(i=0; i < (ws->size) ; i++)
+            for(i=0; i < (int)(ws->size) ; i++)
             {
-               j = (i + start)%(ws->size);
+               j = (i + start)%(int)(ws->size);
                if ( !(use_stdout & WCD_STDOUT_DUMP) )
                   printf("%2d ",k);
                k++;
 
-               printf("%s",ws->dir[j]);
+               wcd_printf("%s",ws->dir[j]);
                if (j == ws->current)
                   printf(" *");
                printf("\n");
@@ -1478,13 +1537,13 @@ int display_list_stdout(nameset list,WcdStack ws, int perfect, int use_stdout)
             printf(_("\nPlease choose one (<Enter> to abort): "));
             i = wcd_get_int();
 
-            if (( i <=0)||(i > ws->size)) /* fail */
+            if (( i <=0)||(i > (int)ws->size)) /* fail */
             {
               return(WCD_ERR_LIST);
              }
             else    /* succes */
             {
-              i = ( i - 1 + start)%(ws->size);
+              i = ( i - 1 + start)%(int)(ws->size);
               ws->current = i;
               return(i);
             }
