@@ -8,7 +8,7 @@ Author: Erwin Waterlander
 ======================================================================
 = Copyright                                                          =
 ======================================================================
-Copyright (C) 2002-2011 Erwin Waterlander
+Copyright (C) 2002-2012 Erwin Waterlander
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,7 +36,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 #include "config.h"
 
-#if defined(UNIX) || defined(DJGPP) || defined(__OS2__)
+#include <stdlib.h>
+#include <sys/stat.h>
+#ifdef _MSC_VER
+#define S_ISDIR( m )    (((m) & _S_IFMT) == _S_IFDIR)
+#endif
+#if defined(UNIX) || defined(__DJGPP__) || defined(__OS2__)
 # include <unistd.h>
 #endif
 #include "dosdir.h"
@@ -45,11 +50,11 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include <string.h>
 #include <errno.h>
 
-#if (defined(WIN32) && defined(WCD_UNICODE))
+#if (defined(_WIN32) && defined(WCD_UNICODE))
 #include <wchar.h>
 #endif
 
-#if (defined(WIN32) || defined(__CYGWIN__))
+#if (defined(_WIN32) || defined(__CYGWIN__))
 
 /* WIN32:  Use WIN32 API to support UNC paths.
    CYGWIN: Use WIN32 API only to scan servers and find shared
@@ -64,10 +69,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #include "wfixpath.h"
 #include "dosdir.h"
 #include <string.h>
+#include "display.h"
 #include <windows.h>
 /* #include <strsafe.h> */
 #include "wcd.h"
-#include "display.h"
 
 
 /*
@@ -218,19 +223,19 @@ void wcd_getshares(char* path, nameset n)
 #endif
 
 
-#if (defined(WIN32) && !defined(__CYGWIN__))
+#if (defined(_WIN32) && !defined(__CYGWIN__))
 /* WIN32, not CYGWIN
    Use WIN32 API */
 
 
-void PrintError(DWORD dw) 
-{ 
+void PrintError(DWORD dw)
+{
     /* Retrieve the system error message for the last-error code */
 
     LPVOID lpMsgBuf;
 
     FormatMessage(
-        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+        FORMAT_MESSAGE_ALLOCATE_BUFFER |
         FORMAT_MESSAGE_FROM_SYSTEM |
         FORMAT_MESSAGE_IGNORE_INSERTS,
         NULL,
@@ -288,7 +293,7 @@ int wcd_isSharePath (char* path)
 char *wcd_getcwd(char *buf, size_t size)
 {
    DWORD err;
-   DWORD dw; 
+   DWORD dw;
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -302,7 +307,7 @@ char *wcd_getcwd(char *buf, size_t size)
 
    if (err == 0)
    {
-     dw = GetLastError(); 
+     dw = GetLastError();
      fprintf(stderr, "%s", _("Wcd: error: Unable to get current working directory: "));
      PrintError(dw);
      return(NULL);  /* fail */
@@ -314,7 +319,7 @@ char *wcd_getcwd(char *buf, size_t size)
 int wcd_chdir(char *buf, int quiet)
 {
    BOOL err;
-   DWORD dw; 
+   DWORD dw;
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -330,7 +335,7 @@ int wcd_chdir(char *buf, int quiet)
    {
       if ( !quiet )
       {
-         dw = GetLastError(); 
+         dw = GetLastError();
          wcd_printf(_("Wcd: error: Unable to change to directory %s: "), buf);
          PrintError(dw);
       }
@@ -343,7 +348,7 @@ int wcd_chdir(char *buf, int quiet)
 int wcd_mkdir(char *buf, int quiet)
 {
    BOOL err;
-   DWORD dw; 
+   DWORD dw;
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -361,7 +366,7 @@ int wcd_mkdir(char *buf, int quiet)
    {
      if ( !quiet )
      {
-       dw = GetLastError(); 
+       dw = GetLastError();
        wcd_printf(_("Wcd: error: Unable to create directory %s: "), buf);
        PrintError(dw);
      }
@@ -372,7 +377,7 @@ int wcd_mkdir(char *buf, int quiet)
 int wcd_rmdir(char *buf, int quiet)
 {
    BOOL err;
-   DWORD dw; 
+   DWORD dw;
 #ifdef WCD_UNICODE
    static wchar_t wstr[DD_MAXPATH];
 
@@ -390,7 +395,7 @@ int wcd_rmdir(char *buf, int quiet)
    {
      if ( !quiet )
      {
-       dw = GetLastError(); 
+       dw = GetLastError();
        wcd_printf(_("Wcd: error: Unable to remove directory %s: "), buf);
        PrintError(dw);
      }
@@ -428,7 +433,7 @@ int wcd_isdir(char *dir, int quiet)
 #ifdef WCD_UTF16
    static wchar_t wstr[DD_MAXPATH];
    BOOL err;
-   DWORD dw; 
+   DWORD dw;
    struct _stat buf;
 #else
    struct stat buf;
@@ -477,7 +482,7 @@ int wcd_isdir(char *dir, int quiet)
       {
         if ( !quiet )
         {
-          dw = GetLastError(); 
+          dw = GetLastError();
           wcd_printf("Wcd: %s: ", dir);
           PrintError(dw);
         }
@@ -509,7 +514,7 @@ int wcd_isdir(char *dir, int quiet)
 
   /*  Use POSIX API  */
 
-#if defined(UNIX) || defined(DJGPP) || defined(__OS2__)
+#if defined(UNIX) || defined(__DJGPP__) || defined(__EMX__)
 
 int wcd_mkdir(char *buf, mode_t m, int quiet)
 {
